WishlistAtlasLoot =  {}

local L = AceLibrary('AceLocale-2.2'):new('Wishlist')
local BuildFromAtlasLoot, ExtractAtlasLootItems, ExtractAtlasLootRequirements

function WishlistAtlasLoot:Hook()
	local atlasData = AtlasLoot_Data
	if atlasData then
		if not atlasData.BCKeys then
			LoadAddOn("AtlasLoot_BurningCrusade")
		end
		if not atlasData.PvP80DeathKnight then
			LoadAddOn("AtlasLoot_WrathOfTheLichKing")
		end

		local atlasMenu = BuildFromAtlasLoot(atlasData)
		if atlasMenu then
			Wishlist.Options.args.AtlasMenu = atlasMenu
		end
	end
end

function BuildFromAtlasLoot(atlasData)
  local unitClass = select(2, UnitClass('player'))
  unitClass = string.sub(unitClass, 1, 1) .. string.lower(string.sub(unitClass, 2))
  local LocalUnitClass = UnitClass('player')

  local pvp = {}
  local pve = {}

	-- TODO: Filtered on class/level/...

	-- PvP
  ExtractAtlasLootItems(pvp, L['Outland Reputation'], atlasData['PVP70Rep' .. unitClass])

  ExtractAtlasLootItems(pvp, L['Arena Armor - '] .. LocalUnitClass, atlasData['Arena' .. unitClass]) -- TODO: Handle multi-spec classes
  ExtractAtlasLootItems(pvp, L['Lvl80 PvP - '] .. LocalUnitClass, atlasData['PvP80' .. unitClass]) -- TODO: Handle multi-spec classes

  ExtractAtlasLootItems(pvp, L['Arena S2 - Weapons 1'], atlasData.Arena2Weapons1)
  ExtractAtlasLootItems(pvp, L['Arena S2 - Weapons 2'], atlasData.Arena2Weapons2)
  ExtractAtlasLootItems(pvp, L['Arena S3 - Weapons 1'], atlasData.Arena3Weapons1)
  ExtractAtlasLootItems(pvp, L['Arena S3 - Weapons 2'], atlasData.Arena3Weapons2)
  ExtractAtlasLootItems(pvp, L['Arena S4 - Weapons 1'], atlasData.Arena4Weapons1)
  ExtractAtlasLootItems(pvp, L['Arena S4 - Weapons 2'], atlasData.Arena4Weapons2)
  ExtractAtlasLootItems(pvp, L['Arena S5 - Weapons 1'], atlasData.Arena5Weapons1)
  ExtractAtlasLootItems(pvp, L['Arena S5 - Weapons 2'], atlasData.Arena5Weapons2)

  ExtractAtlasLootItems(pvp, L['Lvl70 Non-Set Epics 1'], atlasData.PvP70NonSet1)
  ExtractAtlasLootItems(pvp, L['Lvl70 Non-Set Epics 2'], atlasData.PvP70NonSet2)
  ExtractAtlasLootItems(pvp, L['Lvl70 Non-Set Epics 3'], atlasData.PvP70NonSet3)
  ExtractAtlasLootItems(pvp, L['Lvl70 Non-Set Epics 4'], atlasData.PvP70NonSet4)
  ExtractAtlasLootItems(pvp, L['Lvl70 Non-Set Epics 5'], atlasData.PvP70NonSet5)

	-- PvE  
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Accessories 1'], atlasData.HardModeAccessories)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Accessories 2'], atlasData.HardModeAccessories2)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Veteran Armor 1'], atlasData.HardModeArena)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Veteran Armor 2'], atlasData.HardModeArena2)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Cloaks'], atlasData.HardModeCloaks)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Cloth'], atlasData.HardModeCloth)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Leather 1'], atlasData.HardModeLeather)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Leather 2'], atlasData.HardModeLeather2)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Mail'], atlasData.HardModeMail)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Plate 1'], atlasData.HardModePlate)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Plate 2'], atlasData.HardModePlate2)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Relics'], atlasData.HardModeRelic)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Fire Prot. Gears'], atlasData.HardModeResist)
  ExtractAtlasLootItems(pve, L['Lvl 70 Instance Token Rewards - Weapons'], atlasData.HardModeWeapons)

	local atlasMenu = {}

	if #pvp > 0 then
		table.insert(atlasMenu, { type = 'group', name = L['PvP'], desc = L['PvP'], order = 1, args = pvp })
	end
	if #pve > 0 then
		table.insert(atlasMenu, { type = 'group', name = L['PvE'], desc = L['PvE'], order = 2, args = pve })
	end

	if #atlasMenu > 0 then
		return { type = 'group', name = L['Import from AtlasLoot'], desc = L['Import from AtlasLoot'], order = 1000, args = atlasMenu }
	end

	return nil
end

function ExtractAtlasLootItems(menuArgs, name, atlasPart)
	local args = {}
	if atlasPart then
		local index = 1
		local _, part
		for _, part in ipairs(atlasPart) do
			local id = part[2]
			if id ~= 0 then
				local name, link, _, _, _, _, _, _, _, texture = GetItemInfo(id)
				if name then
					local requirements = {}
					local atlasReq = part[6]
					if atlasReq then
						local fields = {}
						atlasReq:gsub("[^ ]+", function(c) table.insert(fields, c) end)
						-- TODO: Try everything
						ExtractAtlasLootRequirements(requirements, fields[1], fields[2])
						ExtractAtlasLootRequirements(requirements, fields[3], fields[4])
					end
					-- TODO: Requirements in description
					table.insert(args, { type = 'execute', name = link, desc = name, order = index, icon = texture, func = function() self:AddNewItem({ ItemId = id, Requirements = requirements }, link) end })
					index = index + 1
				end
			end
		end
	end

	if #args > 0 then
		table.insert(menuArgs, { type = 'group', name = name, desc = name, order = #menuArgs + 1, args = args })
	end
end

function ExtractAtlasLootRequirements(requirements, req1, req2)
  if req1 and req2 then
    if req2 == '#faction#' or req2 == '#' .. strlower(UnitFactionGroup('player')) .. '#' then
      table.insert(requirements, { Type = 'Honor', Honor = tonumber(req1) })
    elseif req2 == '#arena#' then
      table.insert(requirements, { Type = 'Arena', ArenaPoints = tonumber(req1) })
    elseif req2 == '#av#' then
      table.insert(requirements, { Type = 'Item', Name = L['Alterac Valley Mark of Honor'], Required = tonumber(req1) })
    elseif req2 == '#ab#' then
      table.insert(requirements, { Type = 'Item', Name = L['Arathi Basin Mark of Honor'], Required = tonumber(req1) })
    elseif req2 == '#wsg#' then
      table.insert(requirements, { Type = 'Item', Name = L['Warsong Gulch Mark of Honor'], Required = tonumber(req1) })
    elseif req2 == '#eos#' then
      table.insert(requirements, { Type = 'Item', Name = L['Eye of the Storm Mark of Honor'], Required = tonumber(req1) })
     elseif req2 == '#heroic#' then
      table.insert(requirements, { Type = 'Item', Name = L['Badge of Justice'], Required = tonumber(req1) })
    end
    -- TODO: Reputation...
  end
end